@@ -3,6 +3,7 @@ from __future__ import print_function
 from collections import namedtuple, defaultdict
 import copy
 import json
+from operator import itemgetter
 from os.path import basename
 import struct
 
@@ -555,6 +556,8 @@ class ShardReader(object):
     """
     label, return_multiple = toiter(label, is_iter=True)
     label = set(( int(l) for l in label))
+    if not label:
+      return {}
 
     cached = {}
     if self.cache.enabled:
@@ -586,18 +589,42 @@ class ShardReader(object):
       for ext in exists.values()
     )
 
+    # Requesting many individual shard chunks is slow, but due to z-ordering
+    # we might be able to combine adjacent byte ranges. Especially helpful
+    # when downloading entire shards!
+    bundles = []
+    for chunk in sorted(files, key=itemgetter("path", "start")):
+      if not bundles or (chunk['path'] != bundles[-1]['path']) or (chunk['start'] != bundles[-1]['end']):
+        bundles.append(dict(content=None, subranges=[], **chunk))
+      else:
+        bundles[-1]['end'] = chunk['end']
+
+      bundles[-1]['subranges'].append({
+          'start': chunk['start'],
+          'length': chunk['end'] - chunk['start'],
+          'slices': slice(chunk['start'] - bundles[-1]['start'], chunk['end'] - bundles[-1]['start'])
+      })
+
+
     full_path = self.meta.join(self.meta.cloudpath, path)
-    remote_files = CloudFiles(full_path, progress=progress, green=self.green).get(files)
+    bundles_resp = CloudFiles(full_path, progress=progress, green=self.green).get(bundles)
+
+    # Responses are not guaranteed to be in order of requests
+    bundles_resp = { (r['path'], r['byte_range']): r for r in bundles_resp }
 
     binaries = {}
-    for res in remote_files:
-      if res['error']:
-        raise res['error']
-      start, end = res['byte_range']
-      key = (res['path'], start, end - start)
-      lbl = key_label[key]
-      binaries[lbl] = res['content']
-    del remote_files
+    for bundle_req in bundles:
+      bundle_resp = bundles_resp[(bundle_req['path'], (bundle_req['start'], bundle_req['end']))]
+      if bundle_resp['error']:
+        raise bundle_resp['error']
+
+      for chunk in bundle_req['subranges']:
+        key = (bundle_req['path'], chunk['start'], chunk['length'])
+        lbl = key_label[key]
+        binaries[lbl] = bundle_resp['content'][chunk['slices']]
+
+    del bundles
+    del bundles_resp
 
     if self.spec.data_encoding != 'raw':
       for filepath, binary in tqdm(binaries.items(), desc="Decompressing", disable=(not progress)):
@@ -728,8 +755,7 @@ def synthesize_shard_file(spec, label_group, progress=False, presorted=False):
       continue
 
     minishard_index = np.zeros( (3, len(labels)), dtype=np.uint64, order='C')
-    minishard = b''
-    
+    minishard_components = []
     # label and offset are delta encoded
     last_label = 0
     for i, label in enumerate(labels):
@@ -740,10 +766,11 @@ def synthesize_shard_file(spec, label_group, progress=False, presorted=False):
       minishard_index[0, i] = label - last_label
       minishard_index[1, i] = 0 # minishard_index[2, i - 1]
       minishard_index[2, i] = len(binary)
-      minishard += binary
+      minishard_components.append(binary)
       last_label = label
       del minishardgrp[label]
-    
+
+    minishard = b"".join(minishard_components)
     minishardnos.append(minishardno)
     minishard_indicies.append(minishard_index) 
     minishards.append(minishard)
